"""
Configs migration

Generation datetime: 2022-12-14 11:09:01.983335
"""

from typing import List

from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)

mh: DatabaseMigrationHelper

REVISION = "657a9024"
DOWN_REVISION = "bf637ca5"


def settingUpgradeFunction(setting: dict):
    del setting["liveness"]
    return setting


def settingDowngradeFunction(setting: dict):
    setting["liveness"] = False
    return setting


limitationsToModify: List[LimitationToModify] = [
    LimitationToModify(
        limitationName="ADDITIONAL_SERVICES_USAGE",
        oldValidationSchema={
            "type": "object",
            "properties": {
                "luna_sender": {"type": "boolean"},
                "luna_events": {"type": "boolean"},
                "luna_tasks": {"type": "boolean"},
                "indexed_matcher": {"type": "boolean"},
                "luna_matcher_proxy": {"type": "boolean"},
                "liveness": {"type": "boolean"},
            },
            "required": [
                "luna_sender",
                "luna_events",
                "luna_tasks",
                "indexed_matcher",
                "luna_matcher_proxy",
                "liveness",
            ],
            "description": "whether to use nested optional services",
            "additionalProperties": False,
        },
        newValidationSchema={
            "type": "object",
            "properties": {
                "luna_sender": {"type": "boolean"},
                "luna_events": {"type": "boolean"},
                "luna_tasks": {"type": "boolean"},
                "indexed_matcher": {"type": "boolean"},
                "luna_matcher_proxy": {"type": "boolean"},
            },
            "required": [
                "luna_sender",
                "luna_events",
                "luna_tasks",
                "indexed_matcher",
                "luna_matcher_proxy",
            ],
            "description": "whether to use nested optional services",
            "additionalProperties": False,
        },
        oldDefaultValue={
            "luna_sender": True,
            "luna_events": True,
            "luna_tasks": True,
            "indexed_matcher": False,
            "luna_matcher_proxy": False,
            "liveness": False,
        },
        newDefaultValue={
            "luna_sender": True,
            "luna_events": True,
            "luna_tasks": True,
            "indexed_matcher": False,
            "luna_matcher_proxy": False,
        },
        oldDescription="whether to use nested optional services",
        newDescription="whether to use nested optional services",
        oldGroupNames=["luna-platform"],
        newGroupNames=["luna-platform"],
        settingUpgradeFunction=settingUpgradeFunction,
        settingDowngradeFunction=settingDowngradeFunction,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=[
            "luna-admin",
            "luna-api",
            "luna-handlers",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-tasks",
        ],
        newServices=[
            "luna-admin",
            "luna-api",
            "luna-handlers",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-tasks",
        ],
    ),
]

externalLimitationsToModify: List[ExternalLimitationToModify] = []


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to remove
    mh.removeLimitation(limitationName="LIVENESSV1_THRESHOLD")
    mh.removeLimitation(limitationName="LIVENESS_ADDRESS")
    mh.removeLimitation(limitationName="LIVENESS_PREDICTION_TIMEOUTS")

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to add
    mh.addLimitation(
        limitationName="LIVENESSV1_THRESHOLD",
        validationSchema={"type": "number", "exclusiveMinimum": 0},
        defaultValue=0.5,
        description="Threshold for the likelihood that a person is alive for liveness v.1",
        services=["luna-api"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LIVENESS_ADDRESS",
        validationSchema={
            "type": "object",
            "properties": {
                "origin": {
                    "type": "string",
                    "pattern": "^http(s)?:\\/\\/[^\\/:]+(:\\d+)?$",
                    "example": "http://127.0.0.1:5051",
                }
            },
            "required": ["origin"],
            "additionalProperties": False,
        },
        defaultValue={"origin": "http://localhost:8080"},
        description="Http settings for liveness",
        services=["luna-admin", "luna-api"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LIVENESS_PREDICTION_TIMEOUTS",
        validationSchema={
            "type": "object",
            "properties": {
                "request": {"type": "number", "exclusiveMinimum": 0},
                "connect": {"type": "number", "exclusiveMinimum": 0},
                "sock_connect": {"type": "number", "exclusiveMinimum": 0},
                "sock_read": {"type": "number", "exclusiveMinimum": 0},
            },
            "description": "timeouts in seconds",
            "additionalProperties": False,
        },
        defaultValue={
            "request": 60,
            "connect": 20,
            "sock_connect": 10,
            "sock_read": 20,
        },
        description="timeouts for liveness service",
        services=["luna-api"],
        groupNames=["luna-platform"],
    )
