import configparser
import logging
import os
import sys
from configparser import ParsingError
from pathlib import Path
from typing import Any, List

import helpers
from configs.settings.all_settings import SETTINGS as ALL_SETTINGS

__package__ = helpers.solve_imports(level=3, file_name=__file__)


def get_logger() -> logging.Logger:
    """
    Initialize logger with DEBUG level.
    """
    logger = logging.Logger(name="validate_config", level=logging.DEBUG)
    stream_handler = logging.StreamHandler()
    stream_handler.setFormatter(logging.Formatter("%(asctime)s:%(levelname)s: %(message)s"))
    logger.addHandler(stream_handler)
    return logger


class ConfigFileValidator:
    """
    Class for validation of config file config.conf for all services
    """

    def __init__(self, service_name: str, config_path: str) -> None:
        self.service_name = service_name
        self.config_path = config_path

    def _get_service_config_class(self) -> Any:
        """
        Get service config class instance for config validation
        Returns:
            service config class
            error message if validation failed
        """
        if self.service_name == "luna-streams":
            sys.path.append(str(Path.cwd() / "luna-streams"))
            from streams_configs.streams_configs.services.streams import SettingsStreams

            settings = SettingsStreams()
        else:
            from ..service_config import ServiceSettings

            settings = ServiceSettings()

        return settings

    def _get_vars_from_config(self) -> List[str]:
        """
        Get list of config file sections/variables
        """
        config = configparser.ConfigParser()
        config.read(self.config_path)
        sections = config.sections()

        vars = []
        for section in sections:
            vars.append(section if "." not in section else section.split(".")[0])
            if section == "OTHER":
                items = config.items(section)
                for item in items:
                    vars.append(item[0].upper())

        return vars

    def validate_config_sections(self) -> None:
        """
        Validating the presence of required sections/variables in a config file
        """
        required_vars = []
        for key, value in ALL_SETTINGS.items():
            if self.service_name in value.services:
                if self.service_name == "luna-configurator" and key == "LUNA_CONFIGURATOR":
                    continue
                required_vars.append(key)

        config_vars = self._get_vars_from_config()
        if missing_vars := (set(required_vars) - set(config_vars)):
            raise KeyError(
                f"[SECTIONS VALIDATION ERROR] There are missing sections/variables in config file: {missing_vars}"
            )

    def validate_config_values(self) -> None:
        """
        Validating the values of required sections/variables in a config file
        """
        settings = self._get_service_config_class()
        try:
            settings.load(configFile=self.config_path, configuratorAddress=None, tags=None)
        except (KeyError, ValueError, ParsingError) as ex:
            raise ValueError(f"[VALUE VALIDATION ERROR] There are invalid value in config file. {str(ex)}")


if __name__ == "__main__":
    logger = get_logger()

    app_folder = helpers.get_service_path()
    service_name = helpers.get_service_name()
    config_path = app_folder / "configs" / "config.conf"

    if not os.path.exists(config_path):
        raise FileNotFoundError(f"Config file '{config_path}' not found")

    validator = ConfigFileValidator(service_name, config_path)
    validator.validate_config_sections()
    validator.validate_config_values()

    logger.info(f"Config for service {service_name} validated")
